package com.headcaselabs.util;

/**
 * Java class to generate a MicroID, conforming to verison 0.2 of the
 * specification at: http://microid.org/
 *
 * Example usage:
 *
 *    com.headcaselabs.util.MicroId microid =
 *        com.headcaselabs.util.MicroId.getInstance()
 *
 *    String microid_hash =
 *        microid.generate("mailto:foo@example.com", "http://example.com/foo");
 *
 * To verify, one may test these generated values against results
 * obtained from another implementation such the JavaScript code at:
 * http://pajhome.org.uk/crypt/md5/
 *
 * Copyright (C)2007, HeadCase Humanufacturing, Inc.  Released under
 * the Creative Commons Attribution-Share Alike 2.5 License:
 * http://creativecommons.org/licenses/by-sa/2.5/
 *
 * 2007-01-02 Paco NATHAN <paco@headcaselabs.com>
 */

public class
    MicroId
{
    public static final char[] HEXDIGIT =
	{ '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };

    public static final String DEFAULT_ALGORITHM = "SHA-1";
    public static final String DEFAULT_METADATA = "sha1";

    protected java.security.MessageDigest digester = null;
    protected String metadata = null;


    /**
     * Constructor.
     */

    protected
	MicroId (String algorithm)
	throws java.security.NoSuchAlgorithmException
    {
	this.digester = java.security.MessageDigest.getInstance(algorithm);
	this.metadata = algorithm.toLowerCase().replace("-", "");
    }


    /**
     * Factory method which defaults to using "SHA-1".
     */

    public static MicroId
	getInstance ()
	throws java.security.NoSuchAlgorithmException
    {
	return new MicroId(DEFAULT_ALGORITHM);
    }


    /**
     * Factory method which specifies the hashing algorithm.
     */

    public static MicroId
	getInstance (String algorithm)
	throws java.security.NoSuchAlgorithmException
    {
	return new MicroId(algorithm);
    }


    /**
     * Generate a hash value based on the MicroID microformat.
     */

    public String
	generate (String identity_uri, String resource_uri)
    {
	StringBuffer buf = new StringBuffer();

	// first, format the metadata

	buf.append(identity_uri.substring(0, identity_uri.indexOf(":")));
	buf.append('+');
	buf.append(resource_uri.substring(0, resource_uri.indexOf(":")));
	buf.append(':');
	buf.append(metadata);

	// then, the "identity" content

	digester.update(identity_uri.getBytes());

	String who = toHexString(digester.digest());

	// next, the "resource" content

	digester.reset();
	digester.update(resource_uri.getBytes());

	String loc = toHexString(digester.digest());

	// put them together, what does it spell?

	digester.reset();
	digester.update(who.getBytes());
	digester.update(loc.getBytes());

	buf.append(':');
	buf.append(toHexString(digester.digest()));

	return buf.toString();
    }


    /**
     * Utility method to convert a byte array into a String in
     * hexadecimal.
     */

    public static String
	toHexString (byte[] bytes)
    {
	StringBuffer buf = new StringBuffer(bytes.length * 2);

	for (int i = 0; i < bytes.length; i++) {
	    byte b = bytes[i];
	    buf.append(HEXDIGIT[(b & 0xf0) >> 4]).append(HEXDIGIT[b & 0x0f]);
	}

	return buf.toString();
    }
}
